/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db;

import org.simantics.utils.Container;

/**
 * A Graph independent reference to a resource. These can be stored between
 * transactions and later used with any {@link RequestProcessor} or
 * {@link AsyncRequestProcessor} that originates from a Session with the same
 * database.
 * 
 * <p>
 * Resources are considered implementation dependent. Therefore
 * <code>Resource</code> instances from different implementations of this
 * database connection API should not be used together.
 * </p>
 * 
 * <p>
 * This interface should not be implemented by clients.
 * </p>
 * 
 * @author Tuukka Lehtonen
 * @see AsyncReadGraph
 * @see ReadGraph
 * @see WriteGraph
 */
public interface Resource extends Container<Resource>, Comparable<Resource> {

    Resource[] NONE = new Resource[0];

    long getResourceId();
    
    int getThreadHash();
    
    boolean isPersistent();
    
    boolean equalsResource(Resource other);

}
